/**
 * This Software is freeware.
 * Feel free to use/modify/distribute.
 */
package de.japhy.tools.configLog4j;

import java.util.HashSet;

/**
 * @author  <A HREF="mailto:tobias@dezulian.de">Tobias Dezulian</A>
 */
public class CategoryNode extends PropagatingNode{
  public CategoryNode(Object userObject) {
    super(userObject);
  }

  public boolean isDirectChild(PropagatingNode possibleFather, PropagatingNode possibleChild){
    String father=possibleFather.getUserObject().toString();
    String child=possibleChild.getUserObject().toString();
    return  (
              (StringHelper.isClassOfPackage(child, father))
              ||
              (StringHelper.isPackageOfPackage(child, father))
            );
  }

  public boolean isChild(PropagatingNode possibleFather, PropagatingNode possibleChild){
    String father=possibleFather.getUserObject().toString();
    String child=possibleChild.getUserObject().toString();
    return  (
              (StringHelper.isClassOfThisOrSubPackage(child, father))
              ||
              (StringHelper.isPackageOfThisOrSubPackage(child, father))
            );
  }
  public void setError(){
      state = new HashSet();
      state.add(ConfigLog4jConfiguration.ERROR);
  }
  public void setWarn(){
      state = new HashSet();
      state.add(ConfigLog4jConfiguration.WARN);
  }
  public void setInfo(){
      state = new HashSet();
      state.add(ConfigLog4jConfiguration.INFO);
  }
  public void setDebug(){
      state = new HashSet();
      state.add(ConfigLog4jConfiguration.DEBUG);
  }

  /**
   * Propagates the fill status of this inner node to all of its children. <p>
   * Works recursively. <p>
   * This method depends upon wheather the nodes in the tree know that they are
   * inner nodes or leaves.
   */
  public void propagateStateRec(){

    int childNum=this.getChildCount();
    CategoryNode childNode;
    for(int i=0; i<childNum; i++){
      childNode=(CategoryNode) this.getChildAt(i);
      if(state.contains(ConfigLog4jConfiguration.FATAL)) childNode.setFatal();
      else if(state.contains(ConfigLog4jConfiguration.ERROR)) childNode.setError();
      else if(state.contains(ConfigLog4jConfiguration.WARN)) childNode.setWarn();
      else if(state.contains(ConfigLog4jConfiguration.INFO)) childNode.setInfo();
      else if(state.contains(ConfigLog4jConfiguration.DEBUG)) childNode.setDebug();
      else if(state.contains(ConfigLog4jConfiguration.NONE)) childNode.setNone();
      if(!childNode.isLeaf()){
        //inner node
        childNode.propagateStateRec();
      }
    }
  }

  /**
   * Propagates the fill status of this node recursively to its parents.
   */
  protected void checkYourselfRec(){
    //create own state as the merger of the states of all my children
    this.state = new HashSet();
    int childNum=this.getChildCount();
    CategoryNode childNode;
    for(int i=0; i<childNum; i++){
      childNode=(CategoryNode) this.getChildAt(i);
      this.state.addAll(childNode.state);
    }
    //tell my dad to do the same
    CategoryNode dad = (CategoryNode)this.getParent();
    if(!dad.isRoot())
      dad.checkYourselfRec();
  }

  public void setFatal(){
      state = new HashSet();
      state.add(ConfigLog4jConfiguration.FATAL);
  }

  public void setNone(){
      state = new HashSet();
      state.add(ConfigLog4jConfiguration.NONE);
  }
}