package de.japhy.tools.configLog4j;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.TreeSet;
import javax.swing.tree.TreePath;
import java.util.StringTokenizer;
import java.util.Iterator;
import java.io.*;
import javax.swing.border.*;
import java.net.URL;
import java.util.Properties;
import javax.swing.tree.*;

/**
 * @author  <A HREF="mailto:tobias@dezulian.de">Tobias Dezulian</A>
 */
public class ConfigLog4j extends JFrame {

    private static final String  VERSION                     = "v1.0.3.2";
    private static final int     MINIMAL_FRAME_WIDTH         = 600;
    private static final int     MINIMAL_FRAME_HEIGHT        = 300;
    private static final Color   BORDER_COLOR                = Color.black;
    private static final String  KEY_CONFIG_DIR              = "dir.config";
    private static final String  KEY_SOURCE_PATH             =
        "file.sourcepath";
    private static final String  KEY_APPENDERS               =
        "category.appender-ref";
    private static final String  KEY_ADDITIVITY              =
        "category.additivity";
    private static final String  KEY_LOG4J_CONFIG_FILE       =
        "file.log4jconfiguration";
    private static final String  KEY_HEADER_INPUT_FILE       =
        "file.headerinputfile";
    private static final String  KEY_TRAILER_INPUT_FILE      =
        "file.trailerinputfile";
    private static final String  KEY_HELP_PATH               = "help.path";
    private static final String  KEY_CONFIG_FILE_FORMAT      =
        "log4j.configfileformat";
    private static final String  LOG4J_CONFIG_FILE_EXTENSION = "l4j";
    private static String        propertyFileName            = null;
    private static StringBuilder stringBuilder               = null;
    private static ConfigLog4j   frame;
    private static JPanel        treePanel;
    private static String        configFileFormat = null;
    private static String[]      sourcePath;
    private static String[]      appenderRefs = null;
    private static String        additivity   = null;
    private static String        htmlHelpPath;
    private static File          configDir;
    private static File          configFile;
    private static File          headerInputFile;
    private static File          trailerInputFile;
    private static StringBuffer  header            = new StringBuffer();
    private static StringBuffer  trailer           = new StringBuffer();
    private static CategoryTree  myTree            = null;
    JPanel                       jPanel1           = new JPanel();
    BorderLayout                 borderLayout1     = new BorderLayout();
    JButton                      generateButton    = new JButton();
    JButton                      browseButton      = new JButton();
    JTextField                   jTextField1       = new JTextField();
    JLabel                       configLabel       = new JLabel();
    JScrollPane                  jScrollPane1      = new JScrollPane();
    JPanel                       jPanel2           = new JPanel();
    JLabel                       jLabel1           = new JLabel();
    JLabel                       jLabel2           = new JLabel();
    JLabel                       jLabel3           = new JLabel();
    JLabel                       jLabel4           = new JLabel();
    JLabel                       jLabel5           = new JLabel();
    JLabel                       jLabel6           = new JLabel();
    JMenuBar                     menuBar           = new JMenuBar();
    JMenu                        configurationMenu =
        new JMenu("configuration");
    JMenu                        treeMenu          = new JMenu("tree");
    JMenu                        showMenu          = new JMenu("show");
    JMenu                        loadMenu          = new JMenu("load");
    ColorPanel                   fatalPanel        =
        new ColorPanel(ConfigLog4jConfiguration.COLOR_FATAL);
    ColorPanel                   errorPanel        =
        new ColorPanel(ConfigLog4jConfiguration.COLOR_ERROR);
    ColorPanel                   warnPanel         =
        new ColorPanel(ConfigLog4jConfiguration.COLOR_WARN);
    ColorPanel                   infoPanel         =
        new ColorPanel(ConfigLog4jConfiguration.COLOR_INFO);
    ColorPanel                   debugPanel        =
        new ColorPanel(ConfigLog4jConfiguration.COLOR_DEBUG);
    ColorPanel                   nonePanel        =
        new ColorPanel(ConfigLog4jConfiguration.COLOR_NONE);

    JFrame        helpFrame      = null;
    GridBagLayout gridBagLayout2 = new GridBagLayout();
    JPanel        jPanel3        = new JPanel();
    JPanel        jPanel4        = new JPanel();
    GridBagLayout gridBagLayout3 = new GridBagLayout();
    BorderLayout  borderLayout2  = new BorderLayout();
    GridBagLayout gridBagLayout1 = new GridBagLayout();

    private class LoadAction extends AbstractAction {

        LoadAction(String text) {
            super(text);
        }

        /**
         * Method actionPerformed
         *
         *
         * @param e
         *
         * @pre
         *
         * @post
         *
         * @invariant
         *
         */
        public void actionPerformed(ActionEvent e) {

            File      file  = new File(e.getActionCommand());
            TreeModel model = null;

            try {
                ObjectInputStream in =
                    new ObjectInputStream(new FileInputStream(file));

                model = (TreeModel) in.readObject();
            } catch (Exception ioe) {
                System.err.println("couldn't read object in");
                ioe.printStackTrace();
            }

            //System.err.println("setting new model");
            String fileName = e.getActionCommand();

            setFrameTitle("..."
                          + fileName
                              .substring(fileName
                                  .lastIndexOf(System
                                      .getProperty("file.separator")
                                          .charAt(0))));
            myTree.setModel(model);
        }
    }

    private class MyFileFilter extends javax.swing.filechooser.FileFilter {

        /**
         * Method accept
         *
         *
         * @param f
         *
         * @return boolean
         *
         * @pre
         *
         * @post
         *
         * @invariant
         *
         */
        public boolean accept(File f) {
            return f.getAbsolutePath().endsWith(LOG4J_CONFIG_FILE_EXTENSION);
        }

        /**
         * Method getDescription
         *
         *
         * @return String
         *
         * @pre
         *
         * @post
         *
         * @invariant
         *
         */
        public String getDescription() {
            return "Log4j configuration files (*."
                   + LOG4J_CONFIG_FILE_EXTENSION + ")";
        }
    }

    private class MyAction extends AbstractAction {

        MyAction(String text) {
            super(text);
        }

        /**
         * Method actionPerformed
         *
         *
         * @param e
         *
         * @pre
         *
         * @post
         *
         * @invariant
         *
         */
        public void actionPerformed(ActionEvent e) {

            //System.err.println("MyAction.action performed !");
            //HELP
            if (e.getActionCommand().equals("help")) {
                System.err.println("help");

                if (helpFrame == null) {
                    helpFrame = new HelpFrame(htmlHelpPath);

                    helpFrame.addWindowListener(new WindowAdapter() {

                        public void windowClosing(WindowEvent e) {
                            helpFrame.setVisible(false);
                        }
                    });

                    //helpFrame.pack();
                    helpFrame.setSize(new Dimension(800, 400));
                }

                helpFrame.setVisible(true);

                //helpFrame.show();
            }

            //SAVE
            if (e.getActionCommand().equals("save")) {

                //System.err.println("save");
                //Create a file chooser
                final JFileChooser fc = new JFileChooser(configDir);

                //allow only log4j config files
                fc.setFileFilter(new MyFileFilter());

                //In response to a button click:
                fc.setDialogTitle("choose a log4j config file name !");

                //fc.setFileFilter(new L4JFilenameFilter());
                int  returnVal = fc.showSaveDialog(ConfigLog4j.this);
                File file      = null;

                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    String path = fc.getSelectedFile().getAbsolutePath();

                    if (!path.endsWith("." + LOG4J_CONFIG_FILE_EXTENSION)) {
                        path = path + "." + LOG4J_CONFIG_FILE_EXTENSION;
                    }

                    file = new File(path);

                    //System.out.println("opening "+file);
                    try {

                        // make sure
                        file.delete();
                        file.createNewFile();

                        ObjectOutputStream out = new ObjectOutputStream(
                            new FileOutputStream(file));

                        out.writeObject(myTree.getModel());
                    } catch (IOException ioe) {
                        System.err.println("couldn't write object out");
                        ioe.printStackTrace();
                    }
                } else {

                    //System.out.println("operation cancelled ");
                }

                //have the load menu updated to display the new file
                setLoadMenu();
            }

            //EXPAND
            if (e.getActionCommand().equals("expand all")) {
                System.err.println("expand all");

                /*
                TreePath[] tp = myTree.getAllNodePaths();
                System.err.println("n: "+tp.length);
                for(int i=0; i<tp.length; i++){
                  myTree.expandPath(tp[i]);
                }
                */
                for (int i = 0; i < myTree.getRowCount(); i++) {
                    myTree.expandRow(i);
                }
            }

            //COLLAPSE
            if (e.getActionCommand().equals("collapse all")) {
                System.err.println("collapse all");

                // other way around
                for (int i = myTree.getRowCount() - 1; i >= 0; i--) {
                    myTree.collapseRow(i);
                }
            }
        }
    }

    private class L4JFilenameFilter implements FilenameFilter {

        /**
         * Method accept
         *
         *
         * @param file
         * @param name
         *
         * @return boolean
         *
         * @pre
         *
         * @post
         *
         * @invariant
         *
         */
        public boolean accept(File file, String name) {
            return name.endsWith(LOG4J_CONFIG_FILE_EXTENSION);
        }
    }

    /**
     * Constructor ConfigLog4j
     *
     *
     * @pre
     *
     * @post
     *
     * @invariant
     *
     */
    public ConfigLog4j() {

        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }

        setLoadMenu();
    }
    void browseButton_actionPerformed(ActionEvent e) {

        //Create a file chooser
        final JFileChooser fc = new JFileChooser(configFile);

        //In response to a button click:
        fc.setDialogTitle("choose log4j config file !");

        int returnVal = fc.showDialog(this, "Done");

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();

            jTextField1.setText(file.toString());

            //System.out.println("opening "+file);
        } else {

            //System.out.println("operation cancelled ");
        }
    }
    void generateButton_actionPerformed(ActionEvent e) {

        //now go and write the config file.
        String st;
        File   outfile = new File(jTextField1.getText());

        //create category entries

        /*
        <category name="org.log4j.xml">
          <priority value="debug" />
          <appender-ref ref="STDOUT" />
        </category>
        */
        Iterator     iter;
        StringBuffer middle = new StringBuffer();

        //fatal
        TreeSet fatalCats;

        iter = myTree.getFatalCategories().iterator();

        middle.append(stringBuilder.constructComment("FATAL CATEGORIES:"));

        while (iter.hasNext()) {
            middle.append(stringBuilder.constructCat((String) iter.next(),
                                                     "fatal", additivity,
                                                     appenderRefs));
        }

        //error
        TreeSet errorCats;

        iter = myTree.getErrorCategories().iterator();

        middle.append(stringBuilder.constructComment("ERROR CATEGORIES:"));

        while (iter.hasNext()) {
            middle.append(stringBuilder.constructCat((String) iter.next(),
                                                     "error", additivity,
                                                     appenderRefs));
        }

        //warn
        TreeSet warnCats;

        iter = myTree.getWarnCategories().iterator();

        middle.append(stringBuilder.constructComment("WARN CATEGORIES:"));

        while (iter.hasNext()) {
            middle.append(stringBuilder.constructCat((String) iter.next(),
                                                     "warn", additivity,
                                                     appenderRefs));
        }

        //info
        TreeSet infoCats;

        iter = myTree.getInfoCategories().iterator();

        middle.append(stringBuilder.constructComment("INFO CATEGORIES:"));

        while (iter.hasNext()) {
            middle.append(stringBuilder.constructCat((String) iter.next(),
                                                     "info", additivity,
                                                     appenderRefs));
        }

        //debug
        TreeSet debugCats;

        iter = myTree.getDebugCategories().iterator();

        middle.append(stringBuilder.constructComment("DEBUG CATEGORIES:"));

        while (iter.hasNext()) {
            middle.append(stringBuilder.constructCat((String) iter.next(),
                                                     "debug", additivity,
                                                     appenderRefs));
        }

        try {

            //write it all out:
            BufferedWriter out       = new BufferedWriter(
                new OutputStreamWriter(new FileOutputStream(outfile)));
            String         outString = header.toString() + middle.toString()
                                       + trailer.toString();

            out.write(outString, 0, outString.length());
            out.close();

            //System.err.println(outString);
        } catch (Exception e2) {}

        URL  log4jGIF  = ClassLoader.getSystemResource("configLog4j.gif");
        Icon log4jIcon = new ImageIcon(log4jGIF);

        JOptionPane.showMessageDialog(
            this, "generation of config file was successful\nexiting...",
            "configLog4j", JOptionPane.INFORMATION_MESSAGE, log4jIcon);
        System.exit(0);
    }
    private String[] getAvailableConfigurations(File dir) {

        File[]   files    = dir.listFiles(new L4JFilenameFilter());
        String[] filesStr = new String[files.length];

        for (int i = 0; i < files.length; i++) {
            filesStr[i] = files[i].getAbsolutePath();

            /*
            +System.getProperty("file.separator")
            +files[i].getName();
            */
        }

        return filesStr;
    }
    /**
     * Method go
     *
     *
     * @pre
     *
     * @post
     *
     * @invariant
     *
     */
    public void go() {

        /*
        System.err.println(ClassLoader.getSystemResource("x.x"));
        System.exit(0);
        */
        this.addWindowListener(new WindowAdapter() {

            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

        //frame.pack();
        frame.setVisible(true);

        //System.out.println(jButton1.getBackground().toString());
    }
    private void jbInit() throws Exception {

        this.getContentPane().setLayout(borderLayout1);
        this.setSize(new Dimension(MINIMAL_FRAME_WIDTH, 400));
        setFrameTitle("<current source>");

        //config menu
        menuBar.setBorder(new BevelBorder(BevelBorder.RAISED));
        configLabel.setHorizontalTextPosition(SwingConstants.LEFT);
        configLabel.setIconTextGap(0);
        jPanel2.setPreferredSize(new Dimension(100, 68));
        jPanel2.setToolTipText("");
        
        jLabel1.setHorizontalAlignment(SwingConstants.CENTER);
        jLabel1.setHorizontalTextPosition(SwingConstants.CENTER);
        jLabel2.setToolTipText("");
        jLabel2.setHorizontalAlignment(SwingConstants.CENTER);
        jLabel2.setHorizontalTextPosition(SwingConstants.CENTER);
        jLabel5.setToolTipText("");
        jLabel5.setHorizontalAlignment(SwingConstants.CENTER);
        jLabel5.setHorizontalTextPosition(SwingConstants.CENTER);
        jLabel3.setHorizontalAlignment(SwingConstants.CENTER);
        jLabel3.setHorizontalTextPosition(SwingConstants.CENTER);
        jLabel4.setHorizontalAlignment(SwingConstants.CENTER);
        jLabel4.setHorizontalTextPosition(SwingConstants.CENTER);
        jLabel6.setHorizontalAlignment(SwingConstants.CENTER);
        jLabel6.setHorizontalTextPosition(SwingConstants.CENTER);
        
        jPanel4.setLayout(gridBagLayout3);
        jPanel3.setLayout(borderLayout2);
        
        generateButton.setAlignmentX((float) 1.0);
        generateButton.setAlignmentY((float) 1.0);
        generateButton.setHorizontalTextPosition(SwingConstants.CENTER);
        configurationMenu.add(loadMenu);
        configurationMenu.add(new MyAction("save"));
        menuBar.add(configurationMenu);

        //tree menu
        treeMenu.add(new MyAction("expand all"));

        MyAction disabledAction = new MyAction("collapse all");

        //disabledAction.setEnabled(false);
        treeMenu.add(disabledAction);
        menuBar.add(treeMenu);

        //show menu
        showMenu.add(new MyAction("help"));
        menuBar.add(showMenu);

        //menuBar.add(new MyAction("blubb"), BorderLayout.EAST );
        this.setJMenuBar(menuBar);
        jPanel1.setMinimumSize(new Dimension(300, 50));
        jPanel1.setPreferredSize(new Dimension(300, 40));
        jPanel1.setLayout(gridBagLayout2);
        generateButton.setText("generate");
        generateButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                generateButton_actionPerformed(e);
            }
        });
        browseButton.setText("browse");
        browseButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                browseButton_actionPerformed(e);
            }
        });
        jTextField1.setPreferredSize(new Dimension(200, 21));
        jTextField1.setToolTipText("");
        jTextField1.setText(configFile.toString());
        configLabel.setHorizontalAlignment(SwingConstants.LEFT);
        configLabel.setText("log4j config file:");

        //treePanel.add(jFileChooser1, null);
        jLabel1.setText("FATAL");
        jLabel2.setText("DEBUG");
        jLabel3.setText("ERROR");
        jLabel4.setText("WARN");
        jLabel5.setText("INFO");
        jLabel6.setText("NONE");

        jPanel2.setLayout(gridBagLayout1);
        this.getContentPane().add(jScrollPane1, BorderLayout.CENTER);
        this.getContentPane().add(jPanel1, BorderLayout.NORTH);
        jPanel1.add(configLabel,
                    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.WEST,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 6, 0, 6), 0, 0));
        jPanel1.add(jTextField1,
                    new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.HORIZONTAL,
                                           new Insets(0, 0, 0, 0), 0, 0));
        jPanel1.add(browseButton,
                    new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 6, 0, 6), 0, 0));
        this.getContentPane().add(jPanel2, BorderLayout.EAST);
        jPanel2.add(jPanel3,
                    new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.SOUTH,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 0, 0, 0), 0, 0));
        jPanel3.add(generateButton, BorderLayout.SOUTH);
        jPanel2.add(jPanel4,
                    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 0, 0, 0), 0, 30));
        jPanel4.add(jLabel1,
                    new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 0, 0, 0), 0, 0));
        jPanel4.add(jLabel3,
                    new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 0, 0, 0), 0, 0));
        jPanel4.add(jLabel4,
                    new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 0, 0, 0), 0, 0));
        jPanel4.add(jLabel5,
                    new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 0, 0, 0), 0, 0));
        jPanel4.add(jLabel2,
                    new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 0, 0, 0), 0, 0));
        jPanel4.add(jLabel6,
                    new GridBagConstraints(1, 5, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 0, 0, 0), 0, 0));
        jPanel4.add(fatalPanel,
                    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 0, 0, 0), 0, 0));
        jPanel4.add(errorPanel,
                    new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 0, 0, 0), 0, 0));
        jPanel4.add(warnPanel,
                    new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 0, 0, 0), 0, 0));
        jPanel4.add(infoPanel,
                    new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 6, 0, 6), 0, 0));
        jPanel4.add(debugPanel,
                    new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 0, 0, 0), 0, 0));
        jPanel4.add(nonePanel,
                    new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.NONE,
                                           new Insets(0, 0, 0, 0), 0, 0));
        treePanel.setBorder(BorderFactory.createLineBorder(BORDER_COLOR, 10));
        jScrollPane1.getViewport().add(treePanel, BorderLayout.CENTER);

        Border b = BorderFactory.createMatteBorder(10, 10, 10, 10,
                                                   Color.cyan);

        this.addComponentListener(new ComponentAdapter() {

            public void componentResized(ComponentEvent e) {

                if (e.getID() == e.COMPONENT_RESIZED) {
                    Dimension dim = e.getComponent().getSize();

                    if (dim.width < MINIMAL_FRAME_WIDTH) {
                        e.getComponent()
                            .setSize(new Dimension(MINIMAL_FRAME_WIDTH,
                                                   dim.height));
                    }

                    if (dim.height < MINIMAL_FRAME_HEIGHT) {
                        e.getComponent()
                            .setSize(new Dimension(dim.width,
                                                   MINIMAL_FRAME_HEIGHT));
                    }
                }
            }
        });
    }
    /**
     * Method main
     *
     *
     * @param args
     *
     * @pre
     *
     * @post
     *
     * @invariant
     *
     */
    public static void main(String[] args) {

        switch (args.length) {

        case 1 : {
            propertyFileName = args[0];

            break;
        }
        default : {
            System.err
                .print("usage: java ConfigLog4j <CONFIGURATION_FILE_LOCATION>");
            System.exit(-1);
        }
        }

        Properties myProps = new Properties();

        try {
            myProps.load(new FileInputStream(new File(propertyFileName)));
        } catch (Exception e) {
            System.err.println("error reading property file" + e.toString());
        }

        int numTokens;

        //get config file format (xml or java properties)
        configFileFormat = myProps.getProperty(KEY_CONFIG_FILE_FORMAT).trim();

        //get source paths
        StringTokenizer stSource =
            new StringTokenizer(myProps.getProperty(KEY_SOURCE_PATH).trim(),
                                "; ");

        numTokens  = stSource.countTokens();
        sourcePath = new String[numTokens];

        for (int i = 0; i < numTokens; i++) {
            sourcePath[i] = stSource.nextToken();

            System.err.println("source tree #" + i + ": " + sourcePath[i]);
        }

        //get appender-refs
        if (myProps.getProperty(KEY_APPENDERS) != null) {
            if (!myProps.getProperty(KEY_APPENDERS).trim()
                    .equalsIgnoreCase("")) {
                StringTokenizer stApp =
                    new StringTokenizer(myProps.getProperty(KEY_APPENDERS)
                        .trim(), "; ");

                numTokens    = stApp.countTokens();
                appenderRefs = new String[numTokens];

                for (int i = 0; i < numTokens; i++) {
                    appenderRefs[i] = stApp.nextToken();

                    System.err.println("appenderRef #" + i + ": "
                                       + appenderRefs[i]);
                }
            }
        }

        {
            if (myProps.getProperty(KEY_ADDITIVITY) != null) {
                if (!myProps.getProperty(KEY_ADDITIVITY).trim()
                        .equalsIgnoreCase("")) {
                    additivity = myProps.getProperty(KEY_ADDITIVITY).trim();
                }
            }
        }

        configDir        =
            new File(myProps.getProperty(KEY_CONFIG_DIR).trim());
        configFile       =
            new File(myProps.getProperty(KEY_LOG4J_CONFIG_FILE).trim());
        headerInputFile  =
            new File(myProps.getProperty(KEY_HEADER_INPUT_FILE).trim());
        trailerInputFile =
            new File(myProps.getProperty(KEY_TRAILER_INPUT_FILE).trim());
        htmlHelpPath     = myProps.getProperty(KEY_HELP_PATH).trim();

        System.out.println("config file format:     \"" + configFileFormat
                           + "\"");
        System.out.println("default config file:    \"" + configFile + "\"");
        System.out.println("header input file:      \"" + headerInputFile
                           + "\"");
        System.out.println("trailer input file:     \"" + trailerInputFile
                           + "\"");

        //switch depending on the config file format
        if (configFileFormat.equalsIgnoreCase("XML")) {
            stringBuilder = new XmlStringBuilder();
        } else if (configFileFormat.equalsIgnoreCase("PROPERTIES")) {
            stringBuilder = new PropertyStringBuilder();
        } else {
            Helper.error("no config file format specified");
        }

        String st;

        try {

            //get header file
            BufferedReader headerIn = new BufferedReader(
                new InputStreamReader(new FileInputStream(headerInputFile)));

            while (true) {
                st = headerIn.readLine();

                if (st == null) {
                    break;
                }

                header.append(st);
                header.append("\n");
            }

            headerIn.close();

            //get trailer file
            BufferedReader trailerIn = new BufferedReader(
                new InputStreamReader(new FileInputStream(trailerInputFile)));

            while (true) {
                st = trailerIn.readLine();

                if (st == null) {
                    break;
                }

                trailer.append(st);
                trailer.append("\n");
            }

            trailerIn.close();
        } catch (EOFException e3) {
            System.err
                .println("I/O Exception reading header or trailer file.");
            System.exit(-1);
        } catch (IOException e3) {
            System.err
                .println("I/O Exception reading header or trailer file.");
            System.exit(-1);
        }

        TreeSet input = new TreeSet();

        for (int i = 0; i < sourcePath.length; i++) {
            DirRecursor dr = new DirRecursor();

            input.addAll(dr.getPaths(new File(sourcePath[i])));
        }

        TreeSet inputObjects = input;

        //URL leafSelectedURL=ClassLoader.getSystemResource("loggedClass.gif");
        myTree    = new CategoryTree(inputObjects);
        treePanel = new JPanel();

        treePanel.add(myTree, null);

        frame = new ConfigLog4j();

        frame.go();

        try {
            Thread.sleep(10000);
        } catch (Exception e) {
            System.err.println("asdfsdf");
        }
    }
    private void setFrameTitle(String configuration) {
        this.setTitle("configLog4j  " + VERSION
                      + "       config based upon: " + configuration);
    }
    private void setLoadMenu() {

        loadMenu.removeAll();

        String[] availableConfigs = getAvailableConfigurations(configDir);

        for (int i = 0; i < availableConfigs.length; i++) {
            loadMenu.add(new LoadAction(availableConfigs[i]));
        }
    }
}
