/**
 * This Software is freeware.
 * Feel free to use/modify/distribute.
 */
package de.japhy.tools.configLog4j;

import java.io.PrintWriter;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.File;
import java.util.TreeSet;

/**
 * Dives down a directory tree and hands back fully qualified class
 * names for all java source files it finds. <p>
 * Decisive is exclusively the file's name and position in the file
 * system - i.e. files are not parsed for package information. <p>
 *
 * @author  <A HREF="mailto:tobias@dezulian.de">Tobias Dezulian</A>
 *
 */
public class DirRecursor {

  public static void main(String[] args) {
  DirRecursor dr = new DirRecursor();
    System.out.println(dr.getPaths(new File("D:/tmp/gebos")));

  }
  /**
   *
   *
   * @param root The root of the dir tree. E.g. <code>"F:/asdf/asdf/classes/toplevelpackage/"</code>
   * @return TreeSet containing the fully qualified names.
   *
   */
  public TreeSet getPaths(File root) {
    TreeSet tree=new TreeSet();
    int lastSeppOfRootPath =  root.getPath().substring(
                                  0, root.getPath().lastIndexOf(File.separatorChar)
                                  ).length()+1;
    getPathsRec(root, root, lastSeppOfRootPath, tree);
    //System.err.println(tree);
    return tree;
  }

    private void getPathsRec(File root, File dir, int lastSeppOfRootPath, TreeSet tree) {

        tree.add(root.getPath()
          .substring(
            root.getPath().lastIndexOf(File.separatorChar)+1
            , root.getPath().length()
          ).replace(File.separatorChar,'.')
        );

        String files[] = dir.list();
        for (int i=0; i<files.length; i++) {
            File file = new File(dir,files[i]);
            if (file.isDirectory()) {
                getPathsRec(root,file,lastSeppOfRootPath,tree);
                tree.add(file.getPath()
                    .substring(lastSeppOfRootPath).replace(File.separatorChar,'.')
                );
            }
            else if (files[i].endsWith(".java") || files[i].endsWith(".class"))
            {
                postTreatFile(file);
                tree.add(file.getPath()
                    .substring(0, file.getPath()
                    .lastIndexOf('.')) //kick ".java" extension
                    .substring(lastSeppOfRootPath).replace(File.separatorChar,'.')
                );
            }
        }
    }
    protected void postTreatFile(File file){
      //System.err.println("buh");
    }
}
