/**
 * This Software is freeware.
 * Feel free to use/modify/distribute.
 */
package de.japhy.tools.configLog4j;

import javax.swing.*;
import javax.swing.plaf.UIResource;
import java.awt.*;
import java.awt.image.BufferedImage;
import javax.swing.plaf.metal.MetalLookAndFeel;

/**
 * Provides folder icons in different colors
 *
 * @author  <A HREF="mailto:tobias@dezulian.de">Tobias Dezulian</A>
 */
public class IconFactory {

    public static Icon getTreeFolderIcon(Color frontColor) {
	return new TreeFolderIcon(frontColor);
    }

    public static class TreeFolderIcon extends FolderIcon16 {
        public TreeFolderIcon(Color frontColor){ super(frontColor); }
        public int getShift() { return -1; }
        public int getAdditionalHeight() { return 2; }
    }

    static private final Dimension folderIcon16Size = new Dimension( 16, 16 );

    public static class FolderIcon16 implements Icon{

        Color frontColor;
        transient Image image;

        public FolderIcon16(Color frontColor){ this.frontColor=frontColor; }

	public void paintIcon(Component c, Graphics g, int x, int y) {
	    if (image == null) {
		image = new BufferedImage(getIconWidth(), getIconHeight(),
					  BufferedImage.TYPE_INT_ARGB);
                Graphics imageG = image.getGraphics();
		paintMe(c,imageG);
		imageG.dispose();

	    }
            g.drawImage(image, x, y+getShift(), null);
	}

	private void paintMe(Component c, Graphics g) {

	    int right = folderIcon16Size.width - 1;
	    int bottom = folderIcon16Size.height - 1;

	    // Draw tab top
	    g.setColor( MetalLookAndFeel.getPrimaryControlDarkShadow() );
	    g.drawLine( right - 5, 3, right, 3 );
	    g.drawLine( right - 6, 4, right, 4 );

	    // Draw folder front
	    //g.setColor( MetalLookAndFeel.getPrimaryControl() );
	    g.setColor(frontColor);
	    g.fillRect( 2, 7, 13, 8 );

	    // Draw tab bottom
	    g.setColor( MetalLookAndFeel.getPrimaryControlShadow() );
	    g.drawLine( right - 6, 5, right - 1, 5 );

	    // Draw outline
	    g.setColor( MetalLookAndFeel.getPrimaryControlInfo() );
	    g.drawLine( 0, 6, 0, bottom );            // left side
	    g.drawLine( 1, 5, right - 7, 5 );         // first part of top
	    g.drawLine( right - 6, 6, right - 1, 6 ); // second part of top
	    g.drawLine( right, 5, right, bottom );    // right side
	    g.drawLine( 0, bottom, right, bottom );   // bottom

	    // Draw highlight
	    g.setColor( MetalLookAndFeel.getPrimaryControlHighlight() );
	    g.drawLine( 1, 6, 1, bottom - 1 );
	    g.drawLine( 1, 6, right - 7, 6 );
	    g.drawLine( right - 6, 7, right - 1, 7 );

	}
        public int getShift() { return 0; }
        public int getAdditionalHeight() { return 0; }

	public int getIconWidth() { return folderIcon16Size.width; }
	public int getIconHeight() { return folderIcon16Size.height + getAdditionalHeight(); }
    }
}