package de.japhy.tools.configLog4j;

import javax.swing.JPanel;
import javax.swing.Icon;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.ImageIcon;
import javax.swing.ToolTipManager;
import java.net.URL;
import java.io.IOException;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JFrame;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Cursor;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.BorderLayout;

import java.awt.event.*;

import java.util.TreeSet;
import java.util.Iterator;
import java.util.HashSet;

/**
 * Represents some node inside a PropagatingTree. <p><p>
 *
 *
 * TEST
 * <ul>
 * <li>One or more text strings that describe the function. These strings
 *     can be used, for example, to display the flyover text for a button
 *     or to set the text in a menu item.
 * <li>One or more icons that depict the function. These icons can be used
 *     for the images in a menu control, or for composite-entries in a more
 *     sophisticated user-interface.
 * <li>The enabled/disabled state of the functionality. Instead of having
 *     to separately disable the menu-item and the toolbar-button, the
 *     application can disable the function that implements this interface.
 *     All components which are registered as listeners for the state-change
 *     then know to disable event-generation for that item and to modify the
 *     display accordingly.
 * </ul>
 *
 * @author  <A HREF="mailto:tobias@dezulian.de">Tobias Dezulian</A>
 */
 public abstract class PropagatingNode extends DefaultMutableTreeNode{

  //possible states (for inner or leaf node)
  protected HashSet state = new HashSet();

  //am i root ?
  private boolean iAmRoot=false;

  //make sure there can only be one root
  private static boolean rootExists=false;

  private Icon someIcon = new SomeIcon();
  private static int rectangleHeight = 3;
  private static int rectangleVerticalGap = 1;
  private static int rectangleLeftX = 2;
  private static int rectangleWidth = 10;

  public class SomeIcon implements Icon, java.io.Serializable{
    private final Dimension iconSize = new Dimension( 14, 23 );
    private final int gap = 6;

    public int getIconWidth() { return iconSize.width; }
    public int getIconHeight() { return iconSize.height+gap; }
    public void paintIcon(Component c, Graphics g, int xx, int yy) {
      int width = iconSize.width - 1;
      int height = iconSize.height - 1;

      if(isLeaf()){
        g.setColor(ConfigLog4jConfiguration.COLOR_ICON_BORDER );
        g.fillOval( width/2, height/2, 8, 8 );
        if(state.contains(ConfigLog4jConfiguration.FATAL))  setColorDependingOnState( ConfigLog4jConfiguration.FATAL, g );
        else if(state.contains(ConfigLog4jConfiguration.ERROR))  setColorDependingOnState( ConfigLog4jConfiguration.ERROR, g );
        else if(state.contains(ConfigLog4jConfiguration.WARN))  setColorDependingOnState( ConfigLog4jConfiguration.WARN, g );
        else if(state.contains(ConfigLog4jConfiguration.INFO))  setColorDependingOnState( ConfigLog4jConfiguration.INFO, g );
        else if(state.contains(ConfigLog4jConfiguration.DEBUG))  setColorDependingOnState( ConfigLog4jConfiguration.DEBUG, g );
        else if(state.contains(ConfigLog4jConfiguration.NONE))  setColorDependingOnState( ConfigLog4jConfiguration.NONE, g );
        g.fillOval( width/2, height/2, 6, 6 );
      }
      else{
        g.setColor( Color.black );
        g.fillRect( 1, 1, width, height );
        g.setColor( ConfigLog4jConfiguration.COLOR_ICON_BORDER );
        g.drawLine( 0, height, width, height );
        g.drawLine( width, 0, width, height );
        g.drawLine( 0, 0, 0, height );
        g.drawLine( 0, 0, width, 0 );

        int y=2;
        //FATAL
        setColorDependingOnState( ConfigLog4jConfiguration.FATAL, g );
        g.fillRect( rectangleLeftX, y, rectangleWidth, rectangleHeight );
        y+=rectangleVerticalGap+rectangleHeight;

        //ERROR
        setColorDependingOnState( ConfigLog4jConfiguration.ERROR, g );
        g.fillRect( rectangleLeftX, y, rectangleWidth, rectangleHeight );
        y+=rectangleVerticalGap+rectangleHeight;

        //WARN
        setColorDependingOnState( ConfigLog4jConfiguration.WARN, g );
        g.fillRect( rectangleLeftX, y, rectangleWidth, rectangleHeight );
        y+=rectangleVerticalGap+rectangleHeight;

        //INFO
        setColorDependingOnState( ConfigLog4jConfiguration.INFO, g );
        g.fillRect( rectangleLeftX, y, rectangleWidth, rectangleHeight );
        y+=rectangleVerticalGap+rectangleHeight;

        //DEBUG
        setColorDependingOnState( ConfigLog4jConfiguration.DEBUG, g );
        g.fillRect( rectangleLeftX, y, rectangleWidth, rectangleHeight );
        y+=rectangleVerticalGap+rectangleHeight;

        //NONE
        setColorDependingOnState( ConfigLog4jConfiguration.NONE, g );
        g.fillRect( rectangleLeftX, -1, rectangleWidth, rectangleHeight );
        y+=rectangleVerticalGap+rectangleHeight;      
      }
    }

    private void setColorDependingOnState( Integer stateItem, Graphics g ){
      if((stateItem==ConfigLog4jConfiguration.FATAL) && state.contains(ConfigLog4jConfiguration.FATAL)){
          g.setColor( ConfigLog4jConfiguration.COLOR_FATAL );
      }else if((stateItem==ConfigLog4jConfiguration.ERROR) && state.contains(ConfigLog4jConfiguration.ERROR)){
          g.setColor( ConfigLog4jConfiguration.COLOR_ERROR );
      }else if((stateItem==ConfigLog4jConfiguration.WARN) && state.contains(ConfigLog4jConfiguration.WARN)){
          g.setColor( ConfigLog4jConfiguration.COLOR_WARN );
      }else if((stateItem==ConfigLog4jConfiguration.INFO) && state.contains(ConfigLog4jConfiguration.INFO)){
          g.setColor( ConfigLog4jConfiguration.COLOR_INFO );
      }else if((stateItem==ConfigLog4jConfiguration.DEBUG) && state.contains(ConfigLog4jConfiguration.DEBUG)){
          g.setColor( ConfigLog4jConfiguration.COLOR_DEBUG );
      }else if((stateItem==ConfigLog4jConfiguration.NONE) && state.contains(ConfigLog4jConfiguration.NONE)){
          g.setColor( ConfigLog4jConfiguration.COLOR_NONE );
      }else g.setColor( ConfigLog4jConfiguration.COLOR_ICON_BACKGROUNG );
    }
  }

  /**
   *
   * @param userObject The method toString() of this object is called to label this
   * node. <p>
   */
  public PropagatingNode(Object userObject) {
    super(userObject);

    // root is the first node created !!!
    if(!rootExists){
      iAmRoot=true;
      rootExists=true;
    }

    //todo
    this.state.add(ConfigLog4jConfiguration.NONE);
  }
  /**
   * Takes care that the new child is added to the tree at an appropriate spot. <p>
   * The current node must be the root for the subtree into which the new node can
   * be placed. <p>
   * This method does NOT depend upon wheather the nodes in the tree know that they are
   * inner nodes or leaves.
   *
   * @param newChild The new child.
   * @return boolean tells wheather an appropriate place for "child" could be found in the subtree of this node
   */
  public void addRec(PropagatingNode newChild){
    boolean foundAPlace=false;

    if(Helper.debug){
      if(!iAmRoot)
        Helper.out("adding to NON-ROOT: "+this.getUserObject().toString()+"  child: "+newChild.getUserObject().toString());
      else
        Helper.out("adding to ROOT: "+this.getUserObject().toString()+"  child: "+newChild.getUserObject().toString());
    }
    //search for newChild's place
      if( (!iAmRoot) && (isDirectChild(this, newChild)) ){
        //newChild is direct child of mine AND i am NOT root
        this.add(newChild);
        foundAPlace = true;
      }else{
        //newChild is in my subtree but not my direct child
        //OR i am root.

        //look for appropriate child of mine to add this child to
        for(int i=0; i<getChildCount(); i++){
          if( isChild( (PropagatingNode) this.getChildAt(i), newChild )){
            //newChild is in subtree of my child at [i]

            //it's this child's descendant - delegate addition !
            foundAPlace=true;
            ((PropagatingNode) this.getChildAt(i)).addRec(newChild);
            break;
          }
        }
        if(!foundAPlace){
          if(iAmRoot)
            this.add(newChild);
          else
            Helper.error("misplaced child:"+newChild.getUserObject().toString());
        }
      }//else
  }//add()
  /**
   * Propagates the fill status of this node recursively to its parents.
   */
  abstract protected void checkYourselfRec();
  public Icon getIcon(){
    return someIcon;
  }
  public HashSet getState(){ return state; }
  public abstract boolean isChild(PropagatingNode possibleFather, PropagatingNode possibleChild);
  public abstract boolean isDirectChild(PropagatingNode possibleFather, PropagatingNode possibleChild);
  /**
   * represents this node an inner node (i.e. not a leaf) ?
   */
  public boolean isInner(){
    return !this.isLeaf();
  }
  /**
   * Propagates the fill status of this inner node to all of its children. <p>
   * Works recursively. <p>
   * This method depends upon wheather the nodes in the tree know that they are
   * inner nodes or leaves.
   */
  abstract protected void propagateStateRec();
}
