/**
 * This Software is freeware.
 * Feel free to use/modify/distribute.
 */
package de.japhy.tools.configLog4j;

import javax.swing.Icon;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.ImageIcon;
import javax.swing.ToolTipManager;
import java.net.URL;
import java.io.IOException;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JFrame;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Cursor;
import java.awt.Component;
import java.awt.event.*;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;

import java.util.TreeSet;
import javax.swing.tree.TreeModel;
import java.util.Iterator;
import java.util.Enumeration;

/**
 * @author  <A HREF="mailto:tobias@dezulian.de">Tobias Dezulian</A>
 */
public abstract class PropagatingTree extends JTree{

  protected JTree tree=null;
  //protected PropagatingNode root=null;

  /*
  //override superclass' method to keep cached root up to date
  //all derived classes MUST override this method again !
  public void setModel(TreeModel newModel){
    super.setModel(newModel);
    Object rootObject = this.getModel().getRoot();
    if (rootObject.getClass().isInstance(PropagatingNode.class)) root = (PropagatingNode) rootObject;
  }
  */

  public PropagatingTree(TreeSet inputObjects) {
    super();
    PropagatingNode root; //only temporary due to caching problems.
    root = createNode("ROOT");

    this.setBackground(ConfigLog4jConfiguration.COLOR_BACKGROUND);

    //for easier reference from inner classes
    this.tree=this;

    //incrementally add all nodes to the tree
    Iterator iter=inputObjects.iterator();
    PropagatingNode pn=null;
    int count=0;
    while(iter.hasNext()){
      count++;
      Object obj = iter.next();
      pn = createNode(obj);
      root.addRec(pn);
    }
    Helper.out("added "+count+" nodes to the tree !");
    this.toString();
    if(Helper.debug){
      System.out.println("the nodes:");
      //print tree for debug purposes
      Helper.printNodes(root);
      System.out.println("----------");
    }

    this.setRootVisible(false);

    DefaultTreeModel dtm = new DefaultTreeModel(root);
    this.setSelectionModel(null);
    this.setModel(dtm);
    this.setVisibleRowCount(10);
    this.updateUI();
    this.validate();
    this.validateTree();
    this.treeDidChange();
    this.makeVisible(this.getPathForRow(13));
    this.setScrollsOnExpand(false);
    this.setShowsRootHandles(true);

    //set renderer
    this.setCellRenderer(new PropagatingRenderer());

    //Enable tool tips.
    ToolTipManager.sharedInstance().registerComponent(this);
  }

  //test
  public PropagatingNode getRoot(){
    return (PropagatingNode) this.getModel().getRoot();
  }

  protected abstract PropagatingNode createNode(Object obj);

  public TreePath[] getAllNodePaths(){
    Enumeration enum = ((PropagatingNode) this.getModel().getRoot()).breadthFirstEnumeration();
    int i=0;
    while(enum.hasMoreElements()){
      DefaultMutableTreeNode n = (DefaultMutableTreeNode) enum.nextElement();
      i++;
    }
    TreePath[] tp = new TreePath[i];
    i=0;
    while(enum.hasMoreElements()){
      DefaultMutableTreeNode n = (DefaultMutableTreeNode) enum.nextElement();
      tp[i]=new TreePath( n.getPath() );
      i++;
    }

    return tp;

  }

}